@@ -1,90 +1,136 @@
-# @(#)$Id: Admin.pm 1181 2012-04-17 19:06:07Z pjf $
+# @(#)$Id: Admin.pm 1320 2013-07-31 17:31:20Z pjf $
 
 package CatalystX::Usul::Admin;
 
-use strict;
-use warnings;
-use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 1181 $ =~ /\d+/gmx );
-use parent qw(CatalystX::Usul::Programs);
+use namespace::sweep;
+use version; our $VERSION = qv( sprintf '0.8.%d', q$Rev: 1320 $ =~ /\d+/gmx );
 
 use CatalystX::Usul::Constants;
-use CatalystX::Usul::Functions qw(arg_list class2appdir throw untaint_path);
+use CatalystX::Usul::Functions qw( class2appdir logname emit throw
+                                   untaint_cmdline untaint_identifier );
 use Class::Null;
-use MRO::Compat;
-use File::MailAlias;
-use File::Find      qw(find);
-use List::Util      qw(first);
-use English         qw(-no_match_vars);
-use IO::Interactive qw(is_interactive);
-use Scalar::Util    qw(weaken);
+use English                    qw( -no_match_vars );
+use File::DataClass::Types     qw( ArrayRef Bool Directory HashRef
+                                   LoadableClass Object Path );
+use File::Find                 qw( find );
+use File::Basename             qw( basename dirname );
+use File::Spec::Functions      qw( catfile );
+use IO::Interactive            qw( is_interactive );
+use List::Util                 qw( first );
+use Moo;
+use MooX::Options;
 use TryCatch;
 
-use CatalystX::Usul::Model::UserProfiles;
-use CatalystX::Usul::Roles::Unix;
-use CatalystX::Usul::Users::UnixAdmin;
+extends q(Class::Usul::Programs);
+with    q(CatalystX::Usul::TraitFor::PostInstallConfig);
 
-__PACKAGE__->mk_accessors( qw(secsdir) );
+# Public attributes
+option 'exec_setuid' => is => 'ro', isa => Bool, default => FALSE,
+   documentation     => 'True when executing setuid', short => 'e';
 
-my %CONFIG =
-   ( group_add_cmd   => q(groupadd),
-     group_del_cmd   => q(delgroup),
-     parms           => { update_password => [ FALSE ] },
-     profile_class   => q(CatalystX::Usul::UserProfiles),
-     public_methods  => [ qw(authenticate change_password list_methods) ],
-     role_class      => q(CatalystX::Usul::Roles::Unix),
-     sites_available => [ NUL, qw(etc apache2 sites-available) ],
-     sites_enabled   => [ NUL, qw(etc apache2 sites-enabled) ],
-     server_options  => [ { plack    => q(Plack/Starman) },
-                          { mod_perl => q(Apache/mod_perl) },
-                          { none     => q(None) }, ],
-     user_add_cmd    => q(useradd),
-     user_class      => q(CatalystX::Usul::Users::UnixAdmin),
-     user_del_cmd    => q(deluser),
-     user_mod_cmd    => q(usermod), );
+with    q(Class::Usul::TraitFor::UntaintedGetopts);
 
-sub new {
-   my ($self, @rest) = @_;
+# Private attributes
+has '_commands'        => is => 'ro',   isa => HashRef,
+   default             => sub { { group_add_cmd => q(groupadd),
+                                  group_del_cmd => q(delgroup),
+                                  user_add_cmd  => q(useradd),
+                                  user_del_cmd  => q(deluser),
+                                  user_mod_cmd  => q(usermod), } },
+   reader              => 'commands';
 
-   my $attrs = { config => \%CONFIG, %{ arg_list @rest } };
-   my $new   = $self->next::method( $attrs );
+has '_paragraph'       => is => 'ro',   isa => HashRef,
+   default             => sub { { cl => TRUE, fill => TRUE, nl => TRUE } },
+   reader              => 'paragraph';
 
-   $new->parms  ( { %{ $new->config->{parms} }, %{ $new->parms } }  );
-   $new->secsdir( $new->catdir( $new->config->{vardir}, q(secure) ) );
-   $new->version( $VERSION );
+has '_public_methods'  => is => 'ro',   isa => ArrayRef,
+   default             => sub { [ qw(authenticate dump_user list_methods) ] },
+   reader              => 'public_methods';
+
+has '_secsdir'         => is => 'lazy', isa => Directory,
+   default             => sub { [ $_[ 0 ]->config->vardir, q(secure) ] },
+   coerce              => Directory->coercion, reader => 'secsdir';
+
+has '_sites_available' => is => 'lazy', isa => Path, coerce => Path->coercion,
+   default             => sub { [ NUL, qw(etc apache2 sites-available) ] },
+   reader              => 'sites_available';
+
+has '_sites_enabled'   => is => 'lazy', isa => Path, coerce => Path->coercion,
+   default             => sub { [ NUL, qw(etc apache2 sites-enabled) ] },
+   reader              => 'site_enabled';
+
+has '_server_options'  => is => 'ro',   isa => ArrayRef,
+   default             => sub { [ { plack    => q(Plack/Starman)   },
+                                  { mod_perl => q(Apache/mod_perl) },
+                                  { none     => q(None)            }, ] },
+   reader              => 'server_options';
+
+has '_user_class'      => is => 'lazy', isa => LoadableClass,
+   default             => 'CatalystX::Usul::Users::UnixAdmin',
+   reader              => 'user_class';
+
+has '_users'           => is => 'lazy', isa => Object,
+   default             => sub { $_[ 0 ]->user_class->new( builder => $_[ 0 ] )},
+   reader              => 'users';
+
+# Construction
+around 'BUILDARGS' => sub {
+   my ($next, $self, @args) = @_; my $attr = $self->$next( @args );
+
+   $attr->{mode  } = oct q(027);
+   $attr->{params}->{update_password} ||= [ FALSE ];
+
+   return $attr;
+};
+
+around 'new_with_options' => sub {
+   my ($next, $self, @args) = @_; $ENV{ENV} = NUL; # For taint mode
+
+   my $new = $self->$next( @args ); $new->method or $self->_exit_usage( 0 );
+
+   $REAL_USER_ID != 0 and $ENV{USER} = $ENV{LOGNAME} = getpwuid $REAL_USER_ID;
+
+   $EFFECTIVE_USER_ID  = 0; $REAL_USER_ID  = 0;
+   $EFFECTIVE_GROUP_ID = 0; $REAL_GROUP_ID = 0;
 
    return $new;
-}
+};
 
-sub is_authorised {
-   my $self = shift; my $wanted = $self->method or return FALSE;
+around 'run' => sub {
+   my ($next, $self, @args) = @_; my $method = $self->method || 'unknown';
 
-   first { $wanted eq $_ } @{ $self->config->{public_methods} } and return TRUE;
+   # Running as root not suid root during install
+   if ($self->_is_setuid and not $self->_is_authorised) {
+      $self->error( "Access denied to ${method} for ".logname );
+      exit FAILED;
+   }
 
-   my $user = $self->logname or return FALSE;
+   if (q(user_) eq substr $method, 0, 5) {
+      $self->params->{set_user} = [ substr $method, 5 ];
+      $self->method( q(set_user) );
+   }
 
-   first { __find_method( $wanted, $_ ) } $self->_list_auth_sub_files( $user )
-      or return FALSE;
-   q(user_) eq substr $wanted, 0, 5 or return TRUE;
-   $self->parms->{set_user} = [ substr $wanted, 5 ];
-   $self->method( q(set_user) );
-   return TRUE;
-}
-
-sub is_setuid {
-   my $self = shift; return (stat $self->pathname)[ 2 ] & oct q(04000);
-}
+   return $self->$next( @args );
+};
 
+# Public methods
 sub account_report : method {
-   my $self = shift;
+   my $self = shift; my @argv = @{ $self->extra_argv };
 
-   $self->info( $self->users->user_report( @ARGV ), { no_lead => TRUE } );
+   $self->info( $self->users->user_report( @argv ), { no_lead => TRUE } );
    return OK;
 }
 
 sub authenticate : method {
    my $self = shift;
 
-   try        { $self->users->authenticate( TRUE, @ARGV ) }
+   try {
+      my $user = $self->users->authenticate( @{ $self->extra_argv } );
+
+      $user->has_password_expired
+         and throw error => 'User [_1] password expired',
+                   args  => [ $user->username ], class => 'PasswordExpired';
+   }
    catch ($e) { $self->error( $e->error, { args => $e->args } ); return FAILED }
 
    return OK;
@@ -93,7 +139,7 @@ sub authenticate : method {
 sub create_account : method {
    my $self = shift;
 
-   $self->info( $self->users->create_account( @ARGV ) );
+   $self->info( $self->users->create_account( @{ $self->extra_argv } ) );
    return OK;
 }
 
@@ -101,7 +147,6 @@ sub create_ugrps : method {
    # Create the two groups and one users used by this application
    my $self = shift; my ($cmd, $text);
 
-   my $cfg     = $self->config;
    my $picfg   = $self->read_post_install_config;
    my $user    = $picfg->{process_owner} || $picfg->{owner};
    my $groups  = [ $picfg->{group}, $picfg->{admin_role} ];
@@ -110,7 +155,7 @@ sub create_ugrps : method {
    $text  = 'Use groupadd, useradd, and usermod to create the user ';
    $text .= $picfg->{owner}.' and the groups '.$picfg->{group}.' and ';
    $text .= $picfg->{admin_role};
-   $self->output( $text, $cfg->{paragraph} );
+   $self->output( $text, $self->paragraph );
 
    my $choice  = $self->yorn( 'Create groups and user', $default, TRUE, 0 );
 
@@ -119,29 +164,25 @@ sub create_ugrps : method {
    $text  = 'Which user does the HTTP server run as? This user ';
    $text .= 'will be added to the application group so that it can ';
    $text .= 'access the application\'s files';
-   $self->output( $text, $cfg->{paragraph} );
+   $self->output( $text, $self->paragraph );
 
    $user  = $self->get_line( 'HTTP server user', $user, TRUE, 0 );
 
    # Create the application and support groups
-   $self->_create_app_groups( $cfg->{group_add_cmd}, $groups );
-
+   $self->_create_app_groups( $self->commands->{group_add_cmd}, $groups );
    # Create the user to own the files and support the application
-   $self->_create_app_user( $cfg->{user_add_cmd}, $picfg );
-
+   $self->_create_app_user( $self->commands->{user_add_cmd}, $picfg );
    # Add the process owner user to the application group
-   $self->_add_user_to_group( $cfg->{user_mod_cmd}, $user, $picfg->{group} );
-
-   ($picfg->{uid}, $picfg->{gid}) = $self->get_owner( $picfg );
+   $self->_add_user_to_group( $self->commands->{user_mod_cmd},
+                              $user, $picfg->{group} );
 
    return OK;
 }
 
-sub deploy_server : method {
-   # Redispatch to server specific method
+sub deploy_server : method { # Redispatch to server specific method
    my $self          = shift;
    my $picfg         = $self->read_post_install_config;
-   my $server_opts   = $self->config->{server_options};
+   my $server_opts   = $self->server_options;
    my $deploy_server = $picfg->{deploy_server}
                      ? $picfg->{deploy_server}
                      : (keys %{ $server_opts->[ 0 ] })[ 0 ];
@@ -164,7 +205,7 @@ sub deploy_server : method {
 
    $self->info( "Deploying server ${deploy_server}" );
 
-   my $method = q(_deploy_).$deploy_server.q(_server);
+   my $method = "_deploy_${deploy_server}_server";
 
    my $ref; $ref = $self->can( $method ) and return $self->$ref();
 
@@ -174,29 +215,33 @@ sub deploy_server : method {
 sub delete_account : method {
    my $self = shift;
 
-   $self->info( $self->users->delete_account( @ARGV ) );
+   $self->info( $self->users->delete_account( @{ $self->extra_argv } ) );
+   return OK;
+}
+
+sub dump_user : method {
+   my $self = shift; my $user = shift @{ $self->extra_argv } // logname;
+
+   $self->dumper( $self->users->find_user( $user, TRUE ) );
    return OK;
 }
 
 sub init_suid : method {
-   my $self = shift; my $cfg = $self->config;
-
-   my $picfg = $self->read_post_install_config; my $text;
+   my $self = shift; my $picfg = $self->read_post_install_config; my $text;
 
    $text  = 'Enable wrapper which allows limited access to some root ';
    $text .= 'only functions like password checking and user management. ';
    $text .= 'Necessary if the OS authentication store is used';
-   $self->output( $text, $cfg->{paragraph} );
+   $self->output( $text, $self->paragraph );
 
    $self->yorn( 'Enable suid root', $picfg->{init_suid}, TRUE, 0 ) or return OK;
 
-   my $secd = $self->secsdir; my $path = $self->suid;
+   my $secd = $self->secsdir; my $path = $self->config->suid;
 
    my ($uid, $gid) = $self->get_owner( $picfg );
 
-   $text  = 'Restricting permissions on '.$self->basename( $secd );
-   $text .= " and ".$self->basename( $path );
-
+   $text  = 'Restricting permissions on '.basename( $secd );
+   $text .= ' and '.basename( $path );
    $self->info( $text );
    # Restrict access for these files to root only
    chown 0, $gid, $secd; chmod oct q(02700), $secd;
@@ -206,21 +251,19 @@ sub init_suid : method {
    }
 
    chown 0, $gid, $path; chmod oct q(04750), $path;
-
    return OK;
 }
 
-sub make_default : method {
-   # Create the default version symlink
-   my $self   = shift;
-   my $picfg  = $self->read_post_install_config;
-   my $text   = 'Make this the default version';
+sub make_default : method { # Create the default version symlink
+   my $self = shift; my $picfg = $self->read_post_install_config;
+
+   my $text = 'Make this the default version';
 
    $self->yorn( $text, $picfg->{make_default}, TRUE, 0 ) or return OK;
 
    my $verdir = $self->_unlink_default_link;
 
-   $self->info   ( $self->symlink( NUL, $verdir,    q(default) ) );
+   $self->info   ( $self->file->symlink( NUL, $verdir, q(default) ) );
    $self->run_cmd( [ qw(chown -h), $picfg->{owner}, q(default) ] );
    return OK;
 }
@@ -228,7 +271,7 @@ sub make_default : method {
 sub populate_account : method {
    my $self = shift;
 
-   $self->info( $self->users->populate_account( @ARGV ) );
+   $self->info( $self->users->populate_account( @{ $self->extra_argv } ) );
    return OK;
 }
 
@@ -247,12 +290,22 @@ sub post_install : method {
       $self->_call_post_install_method( $picfg, $key, $val );
    }
 
-   $self->_write_post_install_config( $picfg );
+   $self->write_post_install_config( $picfg );
    return OK;
 }
 
-sub restart_server : method {
-   # Bump start the web server
+sub read_secure : method {
+   my $self    = shift;
+   my $file    = $self->extra_argv->[ 0 ] or return FAILED;
+   my $path    = $self->secsdir->catfile( basename( $file ) );
+   my $msg     = "Admin reading secure file ${file} for ".logname;
+
+   $self->_logger( $msg, q(auth.info), q(read_secure) );
+   emit $path->getlines;
+   return OK;
+}
+
+sub restart_server : method { # Bump start the web server
    my $self    = shift;
    my $picfg   = $self->read_post_install_config;
    my $default = $picfg->{restart_server};
@@ -266,19 +319,19 @@ sub restart_server : method {
 
    ($prog and -x $prog) or return FAILED;
 
-   $self->info( "Server restart, running $cmd" );
+   $self->info( "Server restart, running ${cmd}" );
    $self->run_cmd( $cmd );
    return OK;
 }
 
 sub roles {
-   return shift->_identity->[ 1 ];
+   return $_[ 0 ]->users->roles;
 }
 
 sub roles_update : method {
    my $self = shift;
 
-   $self->output( $self->roles->roles_update( @ARGV ) );
+   $self->output( $self->roles->roles_update( @{ $self->extra_argv } ) );
    return OK;
 }
 
@@ -286,30 +339,29 @@ sub set_owner : method {
    # Now we have created everything and have an owner and group
    my $self = shift; my $picfg = $self->read_post_install_config;
 
-   my $base        = $self->config->{appldir};
+   my $base        = $self->config->appldir;
    my ($uid, $gid) = $self->get_owner( $picfg );
-   my $text        = 'Setting owner '.$picfg->{owner}."($uid) and group ";
-      $text       .= $picfg->{group}."($gid)";
+   my $text        = 'Setting owner '.$picfg->{owner}."(${uid}) and group ";
+      $text       .= $picfg->{group}."(${gid})";
 
    $self->info( $text );
-   chown $uid, $gid, $self->dirname( $base );
+   chown $uid, $gid, dirname( $base );
    find( sub { chown $uid, $gid, $_ }, $base );
    chown $uid, $gid, $base;
    return OK;
 }
 
 sub set_password : method {
-   return shift->update_password( TRUE );
+   return $_[ 0 ]->update_password( TRUE );
 }
 
-sub set_permissions : method {
-   # Set permissions
+sub set_permissions : method { # Set permissions on all files in the app
    my $self = shift; my $picfg = $self->read_post_install_config;
 
-   my $base = $self->config->{appldir}; my $pref = $self->prefix;
+   my $base = $self->config->appldir; my $pref = $self->config->prefix;
 
    $self->info( "Setting permissions on ${base}" );
-   chmod oct q(02750), $self->dirname( $base );
+   chmod oct q(02750), dirname( $base );
 
    find( sub { if    (-d $_)                { chmod oct q(02750), $_ }
                elsif ($_ =~ m{ $pref _ }mx) { chmod oct q(0750),  $_ }
@@ -319,7 +371,7 @@ sub set_permissions : method {
 
    # Make the shared directories group writable
    for my $dir (grep { $_->is_dir }
-                map  { $self->abs_path( $base, $_ ) }
+                map  { $self->file->absolute( $base, $_ ) }
                     @{ $picfg->{create_dirs} }) {
       $dir->chmod( 02770 );
    }
@@ -328,29 +380,25 @@ sub set_permissions : method {
 }
 
 sub set_user : method {
-   my ($self, $user) = @_; my $logname = $self->logname; my $logger;
+   my ($self, $user) = @_; my $logname = logname;
 
-   is_interactive() or throw 'Not interactive';
-   $logger = $self->os->{logger}->{value} or throw 'Logger not specified';
-   -x $logger or throw error => 'Cannot execute [_1]', args => [ $logger ];
-
-   $logname =~ m{ \A ([\w.]+) \z }msx and $logname = $1;
+   $user or throw "No user supplied - ${logname}";
+   is_interactive() or throw "Not interactive - ${logname}";
 
    my $msg  = "Admin suid from ${logname} to ${user}";
-   my $cmd  = [ $logger,  qw(-t suid -p auth.info -i), "${msg}" ];
 
-   $self->run_cmd( $cmd );
+   $self->_logger( $msg, q(auth.info), q(suid) );
 
-   my $path = $self->catfile( $self->config->{binsdir},
-                              $self->prefix.q(_suenv) );
+   my $cfg  = $self->config;
+   my $path = catfile( $cfg->binsdir, $cfg->prefix.q(_suenv) );
+   my @argv = @{ $self->extra_argv };
+   my $cmd  = $argv[ 1 ] || NUL;
 
-   $cmd = $ARGV[ 1 ] || NUL;
-
-   if ($ARGV[ 0 ] and $ARGV[ 0 ] eq q(-)) {
+   if ($argv[ 0 ] and $argv[ 0 ] eq q(-)) {
       # Old style full login, ENV unset, HOME set for new user
       $cmd = "su - ${user}";
    }
-   elsif ($ARGV[ 0 ] and $ARGV[ 0 ] eq q(+)) {
+   elsif ($argv[ 0 ] and $argv[ 0 ] eq q(+)) {
       # Keep ENV as now, set HOME for new user
       $cmd = "su ${user} -c '. ${path} ${cmd}'";
    }
@@ -366,20 +414,23 @@ sub set_user : method {
 sub signal_process : method {
    my $self = shift;
 
-   $self->vars->{pids} = \@ARGV;
-   $self->signal_process_as_root( %{ $self->vars } );
+   $self->ipc->signal_process_as_root( %{ $self->options },
+                                       pids => $self->extra_argv );
    return OK;
 }
 
 sub tape_backup : method {
-   my $self = shift; my ($cmd, $res, $text);
+   my $self = shift; my $cfg = $self->config; my ($cmd, $res, $text);
 
-   $self->info( 'Starting tape backup on '.$self->vars->{device} );
-   $cmd  = $self->catfile( $self->config->{binsdir}, $self->prefix.q(_cli) );
-   $cmd .= SPC.$self->debug_flag;
-   $cmd .= ' -c tape_backup -L '.$self->language;
-   $cmd .= ' -o '.$_.'="'.$self->vars->{ $_ }.'"' for (keys %{ $self->vars });
-   $cmd .= ' -- '.(join q( ), map { '"'.$_.'"' } @ARGV);
+   $self->info( 'Starting tape backup on '.$self->options->{device} );
+   $cmd  = catfile( $cfg->binsdir, $cfg->prefix.q(_cli) );
+   $cmd .= SPC.$self->debug_flag.' -c tape_backup -L '.$self->language;
+
+   for (keys %{ $self->options }) {
+      $cmd .= ' -o '.$_.'="'.$self->options->{ $_ }.'"';
+   }
+
+   $cmd .= ' -- '.(join q( ), map { '"'.$_.'"' } @{ $self->extra_argv });
 
    $self->output( $self->run_cmd( $cmd )->out );
    return OK;
@@ -388,9 +439,12 @@ sub tape_backup : method {
 sub uninstall : method {
    my $self   = shift;
    my $cfg    = $self->config;
-   my $appdir = class2appdir $cfg->{class};
+   my $appdir = class2appdir $cfg->appclass;
    my $picfg  = $self->read_post_install_config;
 
+   $self->yorn( $self->add_leader( 'Are you sure?' ), FALSE, TRUE )
+      or return OK;
+
    if ($picfg->{deploy_server} eq q(plack)) {
       $self->run_cmd( [ q(invoke-rc.d), $appdir, q(stop)   ],
                       { expected_rv => 100 } );
@@ -398,10 +452,10 @@ sub uninstall : method {
       $self->run_cmd( [ q(update-rc.d), $appdir, q(remove) ] );
    }
    elsif ($picfg->{deploy_server} eq q(mod_perl)) {
-      my $link = sprintf '%-3.3d-%s', $cfg->{phase}, $appdir;
+      my $link = sprintf '%-3.3d-%s', $cfg->phase, $appdir;
 
-      $self->io( [ @{ $cfg->{sites_enabled  } }, $link      ] )->unlink;
-      $self->io( [ @{ $cfg->{sites_available} }, $appdir    ] )->unlink;
+      $self->io( [ @{ $self->sites_enabled   }, $link   ] )->unlink;
+      $self->io( [ @{ $self->sites_available }, $appdir ] )->unlink;
       $self->run_cmd( [ q(invoke-rc.d), $appdir, q(restart) ] );
    }
 
@@ -409,15 +463,15 @@ sub uninstall : method {
 
    $io->exists and $io->unlink;
 
-   my $cmd = $self->catfile( $cfg->{binsdir}, $self->prefix.q(_schema) );
+   my $cmd = catfile( $cfg->binsdir, $cfg->prefix.q(_schema) );
 
    $self->run_cmd( [ $cmd, $self->debug_flag, qw(-c drop_database) ],
                    { err => q(stderr), out => q(stdout) } );
-   $self->run_cmd( $self->interpolate_cmd( $cfg->{user_del_cmd },
+   $self->run_cmd( $self->interpolate_cmd( $self->commands->{user_del_cmd},
                                            $picfg->{owner} ) );
-   $self->run_cmd( $self->interpolate_cmd( $cfg->{group_del_cmd},
+   $self->run_cmd( $self->interpolate_cmd( $self->commands->{group_del_cmd},
                                            $picfg->{group} ) );
-   $self->run_cmd( $self->interpolate_cmd( $cfg->{group_del_cmd},
+   $self->run_cmd( $self->interpolate_cmd( $self->commands->{group_del_cmd},
                                            $picfg->{admin_role} ) );
 
    $self->_unlink_default_link;
@@ -425,63 +479,63 @@ sub uninstall : method {
 }
 
 sub untaint_self {
-   my $self = shift; my $cmd = $self->pathname;
+   my $self = shift; my $cmd = $self->config->pathname;
 
-   $cmd .= SPC.$self->debug_flag;
-   $cmd .= ' -e -c "'.$self->method.'"';
+   $cmd .= SPC.$self->debug_flag.' -e -c "'.$self->method.'"';
    $cmd .= ' -L '.$self->language if ($self->language);
    $cmd .= ' -q'                  if ($self->quiet);
-   $cmd .= ' -o '.$_.'="'.$self->vars->{ $_ }.'"' for (keys %{ $self->vars });
-   $cmd .= ' -- '.(join q( ), map { "'".$_."'" } @ARGV);
-   $cmd  = untaint_path $cmd;
 
-   $self->debug and $self->log_debug( $cmd );
+   for (keys %{ $self->options }) {
+      $cmd .= ' -o '.$_.'="'.$self->options->{ $_ }.'"';
+   }
+
+   $cmd .= ' -- '.(join SPC, map { "'".$_."'" } @{ $self->extra_argv });
+   $cmd  = untaint_cmdline $cmd;
+
+   $self->debug and $self->log->debug( $cmd );
    return $cmd;
 }
 
 sub update_account : method {
-   my $self = shift;
+   my $self = shift; my @argv = @{ $self->extra_argv };
 
-   $self->info( $self->users->update_account( @ARGV ) );
+   $self->info( $self->users->update_account( @argv ) );
    return OK;
 }
 
 sub update_mail_aliases : method {
-   my $self = shift;
+   my $self = shift; my @argv = @{ $self->extra_argv };
 
-   $self->output( File::MailAlias->new( @ARGV )->update_as_root );
+   $self->output( $self->users->alias_class->new( @argv )->update_as_root );
    return OK;
 }
 
 sub update_password : method {
-   my $self = shift;
+   my ($self, $flag) = @_; my @argv = @{ $self->extra_argv };
 
-   $self->output( $self->users->update_password( shift, @ARGV ) );
+   $self->output( $self->users->update_password( $flag, @argv ) );
    return OK;
 }
 
 sub update_progs : method {
    my $self    = shift;
    my $cfg     = $self->config;
-   my $path    = $self->io( [ $cfg->{ctrldir}, q(default).$cfg->{conf_extn} ] );
-   my $globals = $self->file_dataclass_schema->load( $path )->{globals}
-      or throw 'No global config from ${path}';
-   my $global  = $globals->{ssh_id} or throw 'No SSH identity from ${path}';
-   my $ssh_id  = $global->{value} or throw 'No SSH identity value from ${path}';
-   my $from    = $ARGV[ 0 ] or throw 'Copy from file path not specified';
-   my $to      = $ARGV[ 1 ] or throw 'Copy to file path not specified';
-   my $cmd     = 'su '.$self->owner." -c 'scp -i ${ssh_id} ${from} ${to}'";
+   my $path    = $self->io( [ $cfg->ctrldir, q(default).$cfg->extension ] );
+   my $globals = $self->file->dataclass_schema->load( $path )->{globals}
+      or throw "No global config from ${path}";
+   my $global  = $globals->{ssh_id} or throw "No SSH identity from ${path}";
+   my $ssh_id  = $global->{value} or throw "No SSH identity value from ${path}";
+   my $from    = $self->extra_argv->[ 0 ]
+      or throw 'Copy from file path not specified';
+   my $to      = $self->extra_argv->[ 1 ]
+      or throw 'Copy to file path not specified';
+   my $cmd     = 'su '.$cfg->owner." -c 'scp -i ${ssh_id} ${from} ${to}'";
 
    $self->info( $self->run_cmd( $cmd )->out );
    return OK;
 }
 
-sub users {
-   return shift->_identity->[ 0 ];
-}
-
 # Private methods
-
 sub _add_user_to_group {
    my ($self, $cmd, $user, $group) = @_; ($cmd and $user and $group) or return;
 
@@ -491,23 +545,24 @@ sub _add_user_to_group {
 }
 
 sub _call_post_install_method {
-   my ($self, $cfg, $key, @args) = @_;
+   my ($self, $picfg, $key, @args) = @_;
 
    if ($key eq q(admin)) {
-      if ($cfg->{post_install}) {
+      if ($picfg->{post_install}) {
          my $ref; $ref = $self->can( $args[ 0 ] ) and $self->$ref();
       }
       else { $self->info( 'Would call '.$args[ 0 ] ) }
    }
    else {
-      my $prog    = $self->prefix.q(_).$key;
+      my $prog    = $self->config->prefix.q(_).$key;
       my $cmd     = [ $prog, $self->debug_flag, q(-c), $args[ 0 ] ];
       my $cmd_str = join SPC, @{ $cmd };
 
-      if ($cfg->{post_install}) {
+      if ($picfg->{post_install}) {
          $self->info( "Running ${cmd_str}" );
-         $cmd->[ 0 ] = $self->abs_path( $self->config->{binsdir}, $cmd->[ 0 ] );
-         $self->run_cmd( $cmd, { err => q(stderr), out => q(stdout) } );
+         $cmd->[ 0 ] = $self->file->absolute( $self->config->binsdir,
+                                              $cmd->[ 0 ] );
+         $self->run_cmd( $cmd, { debug => $self->debug, out => q(stdout) } );
       }
       else { $self->info( "Would run ${cmd_str}" ) }
    }
@@ -519,7 +574,7 @@ sub _create_app_groups {
    my ($self, $cmd, $groups) = @_; ($cmd and $groups) or return;
 
    for my $grp (grep { not getgrnam $_ } @{ $groups }) {
-      $self->info( "Creating group ${grp}" );
+      $self->info   ( "Creating group ${grp}" );
       $self->run_cmd( $self->interpolate_cmd( $cmd, $grp ) );
    }
 
@@ -531,10 +586,11 @@ sub _create_app_user {
 
    ($cmd and $picfg) or return; getpwnam $picfg->{owner} and return;
 
-   my $cfg = $self->config; (my $text = ucfirst $cfg->{class}) =~ s{ :: }{ }gmx;
+   my $cfg  = $self->config;
+  (my $text = ucfirst $cfg->appclass) =~ s{ :: }{ }gmx;
 
-   $picfg->{gecos  } = "$text Support";
-   $picfg->{homedir} = $self->dirname( $cfg->{appldir} );
+   $picfg->{gecos  } = "${text} Support";
+   $picfg->{homedir} = dirname( $cfg->appldir );
 
    $self->info   ( 'Creating user '.$picfg->{owner} );
    $self->run_cmd( $self->interpolate_cmd( $cmd, $picfg ) );
@@ -544,50 +600,44 @@ sub _create_app_user {
 sub _deploy_mod_perl_server {
    my $self = shift;
    my $cfg  = $self->config;
-   my $base = $cfg->{appldir};
-   my $file = class2appdir $cfg->{class};
+   my $base = $cfg->appldir;
+   my $file = class2appdir $cfg->appclass;
    my $from = [ qw(var etc mod_perl.conf) ];
-   my $to   = [ @{ $cfg->{sites_available} }, $file ];
-   my $link = sprintf '%-3.3d-%s', $cfg->{phase}, $file;
+   my $to   = [ @{ $self->sites_available }, $file ];
+   my $link = sprintf '%-3.3d-%s', $cfg->phase, $file;
 
-   $self->info( $self->symlink( $base, $from, $to ) );
+   $self->info( $self->file->symlink( $base, $from, $to ) );
 
-   $from = [ @{ $cfg->{sites_available} }, $file ];
-   $to   = [ @{ $cfg->{sites_enabled  } }, $link ];
-   $self->info( $self->symlink( $base, $from, $to ) );
+   $from = [ @{ $self->sites_available }, $file ];
+   $to   = [ @{ $self->sites_enabled   }, $link ];
+   $self->info( $self->file->symlink( $base, $from, $to ) );
 
    $from = [ NUL, qw(etc apache2 mods-available expires.load) ];
    $to   = [ NUL, qw(etc apache2 mods-enabled   expires.load) ];
-   $self->info( $self->symlink( $base, $from, $to ) );
+   $self->info( $self->file->symlink( $base, $from, $to ) );
    return OK;
 }
 
 sub _deploy_plack_server {
-   my $self = shift; my $file = class2appdir $self->config->{class};
+   my $self = shift;
+   my $cfg  = $self->config;
+   my $file = class2appdir $cfg->appclass;
+   my $path = $self->io( [ NUL, qw(etc default) ] );
+
+   $path->exists or $path->mkpath( 0755 );
+
+   $self->file->absolute( $cfg->appldir, [ qw(var etc etc_default) ] )
+              ->copy    ( [ NUL, qw(etc default), $file ] )
+              ->chmod   ( 0644 );
+
+   $self->file->absolute( $cfg->appldir, [ qw(var etc psgi.sh) ] )
+              ->copy    ( [ NUL, qw(etc init.d), $file ] )
+              ->chmod   ( 0755 );
 
-   $self->abs_path( $self->config->{appldir}, [ qw(var etc psgi.sh) ] )
-        ->copy    ( [ NUL, qw(etc init.d), $file ] )
-        ->chmod   ( 0755 );
    $self->run_cmd ( [ q(update-rc.d), $file, qw(defaults 98 02) ] );
    return OK;
 }
 
-sub _identity {
-   my $self  = shift;
-
-   exists $self->{_identity_cache} and return $self->{_identity_cache};
-
-   my $roles = $self->config->{role_class}->new( $self, {} );
-   my $users = $self->config->{user_class}->new( $self, {} );
-   my $attrs = { path => $self->config->{profiles_path} };
-
-   $users->profiles( $self->config->{profile_class}->new( $self, $attrs ) );
-   $users->roles   ( $roles ); weaken( $users->{roles} );
-   $roles->users   ( $users ); weaken( $roles->{users} );
-
-   return $self->{_identity_cache} = [ $users, $roles ];
-}
-
 sub _interpolate_useradd_cmd {
    my ($self, $cmd, $cfg) = @_;
 
@@ -597,42 +647,51 @@ sub _interpolate_useradd_cmd {
 }
 
 sub _interpolate_usermod_cmd {
-   my ($self, $cmd, $group, $user) = @_;
+   return [ $_[ 1 ], qw(-a -G), $_[ 2 ], $_[ 3 ] ];
+}
 
-   return [ $cmd, qw(-a -G), $group, $user ];
+sub _is_authorised {
+   my $self = shift; my $wanted = $self->method or return FALSE;
+
+   first { $wanted eq $_ } @{ $self->public_methods } and return TRUE;
+
+   my $user = logname or return FALSE;
+
+   first { __find_method( $wanted, $_ ) } $self->_list_auth_sub_files( $user )
+      and return TRUE;
+
+   return FALSE;
+}
+
+sub _is_setuid {
+   return $_[ 0 ]->config->pathname->stat->{mode} & oct q(04000);
 }
 
 sub _list_auth_sub_files {
-   my ($self, $user) = @_;
-
    return grep { $_->is_file }
-          map  { $self->io( [ $self->secsdir, $_.q(.sub) ] ) }
-                 $self->roles->get_roles( $user );
+          map  { $_[ 0 ]->io( [ $_[ 0 ]->secsdir, $_.q(.sub) ] ) }
+                 $_[ 0 ]->roles->get_roles( $_[ 1 ] );
 }
 
-sub _unlink_default_link {
-   my $self   = shift;
-   my $base   = $self->config->{appldir};
-   my $verdir = $self->basename( $base );
+sub _logger {
+   my ($self, $msg, $priority, $tag) = @_; my $logger;
 
-   chdir $self->dirname( $base ); -e q(default) and unlink q(default);
+   $logger = $self->os->{logger}->{value} or throw 'Logger not specified';
+   -x $logger or throw error => 'Cannot execute [_1]', args => [ $logger ];
 
-   return $verdir;
-}
-
-sub _write_post_install_config {
-   my ($self, $picfg) = @_;
-
-   my $cfg  = $self->config;
-   my $path = $self->catfile( $cfg->{ctrldir}, $cfg->{pi_config_file} );
-   my $args = { data => $picfg, path => $path };
-
-   $self->file_dataclass_schema( $cfg->{pi_config_attrs} )->dump( $args );
+   $self->run_cmd( [ $logger, q(-i), q(-p), $priority, q(-t), $tag, "${msg}" ]);
    return;
 }
 
-# Private subroutines
+sub _unlink_default_link {
+   my $base = $_[ 0 ]->config->appldir;
 
+   chdir dirname( $base ); -e q(default) and unlink q(default);
+
+   return basename( $base );
+}
+
+# Private functions
 sub __find_method {
    my ($wanted, $io) = @_; my $hash = HASH_CHAR;
 
@@ -653,46 +712,60 @@ CatalystX::Usul::Admin - Subroutines that run as the super user
 
 =head1 Version
 
-0.7.$Revision: 1181 $
+0.8.$Revision: 1320 $
 
 =head1 Synopsis
 
    # Setuid root program wrapper
    use CatalystX::Usul::Admin;
-   use English qw(-no_match_vars);
 
-   my $prog = CatalystX::Usul::Admin->new( appclass => q(App::Munchies),
-                                           arglist  => q(e) );
+   $ENV{PATH} = q(/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin);
 
-   $EFFECTIVE_USER_ID  = 0; $REAL_USER_ID  = 0;
-   $EFFECTIVE_GROUP_ID = 0; $REAL_GROUP_ID = 0;
+   my $app = CatalystX::Usul::Admin->new_with_options
+      ( appclass => 'YourApp' );
 
-   unless ($prog->is_authorised) {
-      my $text = 'Permission denied to '.$prog->method.' for '.$prog->logname;
+   $app->exec_setuid or exec $app->untaint_self or die "Exec failed\n";
 
-      $prog->error( $text );
-      exit 1;
-   }
-
-   exit $prog->run;
+   exit $app->run;
 
 =head1 Description
 
 Methods called from the setuid root program wrapper
 
+=head1 Configuration and Environment
+
+Defines the following attributes
+
+=over 3
+
+=item C<commands>
+
+Hash ref of OS commands
+
+=item C<exec_setuid>
+
+A boolean which defaults to false. Becomes true after the program has
+re-executed itself
+
+=item C<paragraph>
+
+Hash ref of options that makes output appear in separate paragraphs
+
+=item C<public_methods>
+
+List of methods that do not require authorization
+
+=item C<server_options>
+
+Array ref of deployment options
+
+=back
+
 =head1 Subroutines/Methods
 
-=head2 new
+=head2 BUILDARGS
 
-Constructor
-
-=head2 is_authorised
-
-Is the user authorised to call the method
-
-=head2 is_setuid
-
-Is the program running suid
+Customize the constructor
 
 =head2 account_report
 
@@ -722,6 +795,10 @@ Redispatches the call to one of the server specific methods
 Calls the L<delete account|CatalystX::Usul::Users::UnixAdmin/delete_account>
 method on the user model
 
+=head2 dump_user
+
+Retrieves data for the specified user and dumps it using L<Data::Printer>
+
 =head2 init_suid
 
 Enable the C<setuid> root wrapper?
@@ -736,13 +813,17 @@ to this installation
 =head2 post_install
 
 Runs the post installation methods as defined in the
-L<post installtion config|/read_post_install_config>
+L<post installation config|/read_post_install_config>
 
 =head2 populate_account
 
 Calls the L<populate account|CatalystX::Usul::Users::UnixAdmin/populate_account>
 method on the user model
 
+=head2 read_secure
+
+Reads a key file from the secure directory and prints it to STDOUT
+
 =head2 restart_server
 
 Restarts the web server
@@ -756,6 +837,10 @@ Returns the identity roles object
 Calls the L<roles update|CatalystX::Usul::Roles::Unix/roles_update>
 method on the user model
 
+=head2 run
+
+Modifies the parent method to reject unauthorized calls
+
 =head2 set_owner
 
 Set the ownership of the installed files and directories
@@ -792,13 +877,20 @@ Returns an untainted reconstruction of our own invoking command line
 
 =head2 update_account
 
+Calls C<update_account> in the user model to change attributes of the
+user account
+
 =head2 update_mail_aliases
 
+Updates a mail alias via L<File::MailAlias>
+
 =head2 update_password
 
+Changes a users password by calling C<update_password> on the user model
+
 =head2 update_progs
 
-=head2 users
+Uses C<scp> to copy program files from the specified remote server
 
 =head1 Private Methods
 
@@ -810,19 +902,35 @@ Creates the symlinks necessary to deploy the Apache/mod_perl server
 
 Create the symlink necessary to deploy the Plack server
 
+=head2 _is_authorised
+
+Is the user authorised to call the method
+
+=head2 _is_setuid
+
+Is the program running suid
+
 =head1 Diagnostics
 
 None
 
-=head1 Configuration and Environment
-
-None
-
 =head1 Dependencies
 
 =over 3
 
-=item L<CatalystX::Usul::Programs>
+=item L<CatalystX::Usul::TraitFor::PostInstallConfig>
+
+=item L<Class::Null>
+
+=item L<Class::Usul::Programs>
+
+=item L<CatalystX::Usul::Moose>
+
+=item L<CatalystX::Usul::Constraints>
+
+=item L<IO::Interactive>
+
+=item L<TryCatch>
 
 =back
 
@@ -842,7 +950,7 @@ Peter Flanigan, C<< <Support at RoxSoft.co.uk> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2011 Peter Flanigan. All rights reserved
+Copyright (c) 2013 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
