@@ -1,16 +1,19 @@
-# @(#)$Id: Processes.pm 1181 2012-04-17 19:06:07Z pjf $
+# @(#)$Id: Processes.pm 1319 2013-06-23 16:21:01Z pjf $
 
 package CatalystX::Usul::Controller::Admin::Processes;
 
 use strict;
-use warnings;
-use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 1181 $ =~ /\d+/gmx );
-use parent qw(CatalystX::Usul::Controller);
+use version; our $VERSION = qv( sprintf '0.8.%d', q$Rev: 1319 $ =~ /\d+/gmx );
 
-__PACKAGE__->config( namespace     => q(admin),
-                     process_class => q(Process) );
+use CatalystX::Usul::Moose;
 
-__PACKAGE__->mk_accessors( qw(process_class) );
+BEGIN { extends q(CatalystX::Usul::Controller) }
+
+with q(CatalystX::Usul::TraitFor::Controller::PersistentState);
+
+__PACKAGE__->config( namespace => q(admin), );
+
+has 'process_class' => is => 'ro', isa => Str, default => q(Process);
 
 sub proc_table : Chained(common) Args HasActions {
    my ($self, $c, @args) = @_;
@@ -28,6 +31,8 @@ sub proc_table_signal : ActionFor(proc_table.abort)
    return $c->model( $self->process_class )->signal_process;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -40,13 +45,15 @@ CatalystX::Usul::Controller::Admin::Processes - Process table manipulation
 
 =head1 Version
 
-0.7.$Revision: 1181 $
+0.8.$Revision: 1319 $
 
 =head1 Synopsis
 
-   package MyApp::Controller::Admin;
+   package YourApp::Controller::Admin;
 
-   use base qw(CatalystX::Usul::Controller::Admin);
+   use CatalystX::Usul::Moose;
+
+   BEGIN { extends q(CatalystX::Usul::Controller::Admin) }
 
    __PACKAGE__->build_subcontrollers;
 
@@ -64,8 +71,8 @@ to the processes children
 
 =head2 proc_table_signal
 
-Send the selected processes the specified signal, one of; SIGTERM,
-SIGKILL, or SIGABORT
+Send the selected processes the specified signal, one of; C<SIGTERM>,
+C<SIGKILL>, or C<SIGABORT>
 
 =head1 Diagnostics
 
@@ -81,6 +88,8 @@ None
 
 =item L<CatalystX::Usul::Controller>
 
+=item L<CatalystX::Usul::TraitFor::Controller::PersistentState>
+
 =back
 
 =head1 Incompatibilities
@@ -99,7 +108,7 @@ Peter Flanigan, C<< <Support at RoxSoft.co.uk> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2008 Peter Flanigan. All rights reserved
+Copyright (c) 2013 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
