@@ -1,36 +1,27 @@
-# @(#)$Id: Unix.pm 1181 2012-04-17 19:06:07Z pjf $
+# @(#)$Id: Unix.pm 1319 2013-06-23 16:21:01Z pjf $
 
 package CatalystX::Usul::Roles::Unix;
 
 use strict;
-use warnings;
-use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 1181 $ =~ /\d+/gmx );
-use parent qw(CatalystX::Usul::Roles CatalystX::Usul::IPC);
+use version; our $VERSION = qv( sprintf '0.8.%d', q$Rev: 1319 $ =~ /\d+/gmx );
 
+use CatalystX::Usul::Moose;
 use CatalystX::Usul::Constants;
-use CatalystX::Usul::Functions qw(throw untaint_path);
+use CatalystX::Usul::Functions   qw(throw);
+use CatalystX::Usul::Constraints qw(File);
 use File::UnixAuth;
 use TryCatch;
 
-__PACKAGE__->mk_accessors( qw(backup_extn baseid group_obj group_path
-                              inc _mtime) );
+extends q(CatalystX::Usul::Roles);
 
-sub new {
-   my ($class, $app, $attrs) = @_;
+has 'baseid'     => is => 'ro',   isa => PositiveInt, default => 1000;
 
-   $attrs->{backup_extn} ||= q(.bak);
-   $attrs->{baseid     } ||= 1000;
-   $attrs->{group_path } ||= q(/etc/group);
-   $attrs->{inc        } ||= 1;
-   $attrs->{_mtime     }   = 0;
+has 'group_obj'  => is => 'lazy', isa => Object, init_arg => undef;
 
-   my $new = $class->next::method( $app, $attrs );
+has 'group_path' => is => 'ro',   isa => File, coerce => TRUE,
+   default       => sub { [ NUL, qw(etc group) ] };
 
-   $new->group_path( $new->_build_group_path );
-   $new->group_obj ( $new->_build_group_obj  );
-
-   return $new;
-}
+has 'inc'        => is => 'ro',   isa => PositiveInt, default => 1;
 
 # Factory methods
 
@@ -105,33 +96,21 @@ sub roles_update {
    catch ($e) { $self->lock->reset( k => $key ); throw $e }
 
    $self->lock->reset( k => $key );
-   return "Role update $cmd $fld complete";
+   return "Role update ${cmd} ${fld} complete";
 }
 
 # Private methods
 
 sub _build_group_obj {
-   my $self  = shift;
-
-   return File::UnixAuth->new( ioc_obj     => $self, path => $self->group_path,
+   return File::UnixAuth->new( builder     => $_[ 0 ],
+                               path        => $_[ 0 ]->group_path,
                                source_name => q(group) );
 }
 
-sub _build_group_path {
-   my $self = shift; my $path = untaint_path $self->group_path;
-
-   $path or throw 'Group file path not specified';
-   -f $path or throw error => 'File [_1] not found', args => [ $path ];
-
-   return $path;
-}
-
 sub _get_new_gid {
-   my $self    = shift;
-   my $base_id = $self->baseid;
-   my ($cache) = $self->_load;
-   my $inc     = $self->inc;
-   my $new_id  = $base_id;
+   my $self = shift; my $base_id = $self->baseid; my ($cache) = $self->_load;
+
+   my $inc  = $self->inc; my $new_id = $base_id;
 
    for my $gid (sort { $a <=> $b }
                 map  { $cache->{ $_ }->{id} } keys %{ $cache }) {
@@ -142,18 +121,19 @@ sub _get_new_gid {
 }
 
 sub _load {
-   my $self = shift; my $key = __PACKAGE__.q(::_load);
+   my $self  = shift; my $key = __PACKAGE__.q(::_load);
 
    $self->lock->set( k => $key );
 
-   my $mtime = $self->status_for( $self->group_path )->{mtime};
-   my $updt  = $mtime == $self->_mtime ? FALSE : TRUE;
+   my $cache = $self->cache;
+   my $mtime = $self->group_path->stat->{mtime};
+   my $updt  = delete $cache->{_dirty} ? TRUE : FALSE;
 
-   $self->_mtime( $mtime );
+   $updt or $updt = $mtime == ($cache->{_mtime} || 0) ? FALSE : TRUE;
 
-   $updt or return $self->_cache_results( $key );
+   $updt or return $self->_cache_results( $key ); $cache->{_mtime} = $mtime;
 
-   delete $self->cache->{ $_ } for (keys %{ $self->cache });
+   delete $cache->{ $_ } for (grep { not m{ \A _ }mx } keys %{ $cache });
 
    try {
       my $data = $self->group_obj->load->{group};
@@ -163,16 +143,16 @@ sub _load {
          my $gid        = $group_data->{gid};
          my $users      = $group_data->{members};
 
-         unless (exists $self->cache->{id2name}->{ $gid }) {
-            $self->cache->{id2name}->{ $gid    } = $group;
-            $self->cache->{roles  }->{ $group  } = {
+         unless (exists $cache->{id2name}->{ $gid }) {
+            $cache->{id2name}->{ $gid   } = $group;
+            $cache->{roles  }->{ $group } = {
                id     => $gid,
                passwd => $group_data->{password},
                users  => [] };
          }
 
-         push @{ $self->cache->{roles}->{ $group }->{users} }, @{ $users };
-         push @{ $self->cache->{user2role}->{ $_ } }, $group for (@{ $users });
+         push @{ $cache->{roles}->{ $group }->{users} }, @{ $users };
+         push @{ $cache->{user2role}->{ $_ } }, $group for (@{ $users });
       }
    }
    catch ($e) { $self->lock->reset( k => $key ); throw $e }
@@ -181,10 +161,9 @@ sub _load {
 }
 
 sub _run_as_root {
-   my ($self, $method, @args) = @_;
+   my ($self, $method, @args) = @_; my $suid = $self->config->suid;
 
-   my $cmd = [ $self->suid, ($self->debug ? q(-D) : q(-n)), q(-c),
-               $method, q(--), @args ];
+   my $cmd = [ $suid, $self->debug_flag, q(-c), $method, q(--), @args ];
    my $out = $self->run_cmd( $cmd, { err => q(out) } )->out;
 
    $self->debug and $self->log->debug( $out );
@@ -192,6 +171,8 @@ sub _run_as_root {
    return $out;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -204,7 +185,7 @@ CatalystX::Usul::Roles::Unix - Group management for the Unix OS
 
 =head1 Version
 
-0.7.$Revision: 1181 $
+0.8.$Revision: 1319 $
 
 =head1 Synopsis
 
@@ -212,49 +193,73 @@ CatalystX::Usul::Roles::Unix - Group management for the Unix OS
 
    my $class = CatalystX::Usul::Roles::Unix;
 
-   my $role_obj = $class->new( $attrs, $app );
+   my $role_obj = $class->new( $attr );
 
 =head1 Description
 
 Methods to manipulate the group file which defaults to
-I</etc/group>. This class implements the methods required by it's
-base class
+F</etc/group>. This class implements the methods required by it's
+base class L<CatalystX::Usul::Roles>
+
+=head1 Configuration and Environment
+
+Defines the following attributes;
+
+=over 3
+
+=item C<baseid>
+
+A positive integer which defaults to I<1000>. New id must will be greater
+than or equal to this value
+
+=item C<group_obj>
+
+A lazily constructed object which cannot be passed in the constructor. It
+is an instance of L<File::UnixAuth>
+
+=item C<group_path>
+
+A file which is coerced from the default array ref
+C< [ NUL, qw(etc group) ] >
+
+=item C<inc>
+
+A positive integer which defaults to I<1>. The gap to leave between new
+group ids
+
+=back
 
 =head1 Subroutines/Methods
 
-=head2 new
-
-Constructor
-
 =head2 add_user_to_role
 
-   $role_obj->add_user_to_role( $group, $user );
+   $out = $role_obj->add_user_to_role( $groupname, $username );
 
 Calls the suid root wrapper to add the specified user to the specified
-group
+group. Returns the output from running the command
 
 =head2 create
 
-   $role_obj->create( $group );
+   $out = $role_obj->create( $groupname );
 
 Calls the suid root wrapper to create a new group
 
 =head2 delete
 
-   $role_obj->delete( $group );
+   $out = $role_obj->delete( $groupname );
 
 Calls the suid root wrapper to delete an existing group
 
 =head2 remove_user_from_role
 
-   $role_obj->remove_user_to_role( $group, $user );
+   $out = $role_obj->remove_user_to_role( $groupname, $username );
 
 Calls the suid root wrapper to remove the given user from the
 specified group
 
 =head2 roles_update
 
-   $role_obj->roles_update( $cmd, $field, $user, $group );
+   $out = $role_obj->roles_update( $cmd, $field, $username, $groupname );
 
 Called from the suid root wrapper this is the method that updates the
 group file. The C<$cmd> is either I<add> or I<delete>. The C<$field> is
@@ -264,17 +269,17 @@ either I<user> or I<group>
 
 None
 
-=head1 Configuration and Environment
-
-None
-
 =head1 Dependencies
 
 =over 3
 
 =item L<CatalystX::Usul::Roles>
 
-=item L<Unix::GroupFile>
+=item L<CatalystX::Usul::Moose>
+
+=item L<File::UnixAuth>
+
+=item L<TryCatch>
 
 =back
 
@@ -294,7 +299,7 @@ Peter Flanigan, C<< <Support at RoxSoft.co.uk> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2008 Peter Flanigan. All rights reserved
+Copyright (c) 2013 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
